@@ -21,11 +21,12 @@
 #include "utils.h"
 #include "zathura.h"
 
-#include <girara/commands.h>
+#include <girara-gtk/commands.h>
 #include <girara/datastructures.h>
-#include <girara/session.h>
-#include <girara/settings.h>
-#include <girara/utils.h>
+#include <girara-gtk/session.h>
+#include <girara-gtk/settings.h>
+#include <girara/log.h>
+#include <girara-gtk/utils.h>
 
 bool cmd_bookmark_create(girara_session_t* session, girara_list_t* argument_list) {
   g_return_val_if_fail(session != NULL, false);
@@ -100,7 +101,7 @@ bool cmd_bookmark_list(girara_session_t* session, girara_list_t* GIRARA_UNUSED(a
     return false;
   }
 
-  GString* string = g_string_new(NULL);
+  g_autoptr(GString) string = g_string_new(NULL);
   for (size_t idx = 0; idx != girara_list_size(zathura->bookmarks.bookmarks); ++idx) {
     zathura_bookmark_t* bookmark = girara_list_nth(zathura->bookmarks.bookmarks, idx);
     g_string_append_printf(string, "<b>%s</b>: %u\n", bookmark->id, bookmark->page);
@@ -113,7 +114,6 @@ bool cmd_bookmark_list(girara_session_t* session, girara_list_t* GIRARA_UNUSED(a
     girara_notify(session, GIRARA_INFO, _("No bookmarks available."));
   }
 
-  g_string_free(string, TRUE);
   return true;
 }
 
@@ -181,7 +181,7 @@ bool cmd_jumplist_list(girara_session_t* session, girara_list_t* argument_list)
   }
 
   zathura_jump_t* current_jump = zathura_jumplist_current(zathura);
-  GString* string              = g_string_new(NULL);
+  g_autoptr(GString) string    = g_string_new(NULL);
   for (int i = zathura->jumplist.size - 1; i >= 0 && num_entries > 0; --i, --num_entries) {
     const zathura_jump_t* j = girara_list_nth(zathura->jumplist.list, i);
     g_string_append_printf(string, _("[%d]: page=<b>%2d</b>, x=%f, y=%f %s\n"), i, j->page + 1, j->x, j->y,
@@ -195,7 +195,6 @@ bool cmd_jumplist_list(girara_session_t* session, girara_list_t* argument_list)
     girara_notify(session, GIRARA_INFO, _("No jumplist available."));
   }
 
-  g_string_free(string, TRUE);
   return true;
 }
 
@@ -245,8 +244,7 @@ bool cmd_info(girara_session_t* session, girara_list_t* UNUSED(argument_list)) {
     return false;
   }
 
-  GString* string = g_string_new(NULL);
-
+  g_autoptr(GString) string = g_string_new(NULL);
   for (size_t i = 0; i < LENGTH(meta_fields); i++) {
     for (size_t idx = 0; idx != girara_list_size(information); ++idx) {
       zathura_document_information_entry_t* entry = girara_list_nth(information, idx);
@@ -263,8 +261,6 @@ bool cmd_info(girara_session_t* session, girara_list_t* UNUSED(argument_list)) {
     girara_notify(session, GIRARA_INFO, _("No information available."));
   }
 
-  g_string_free(string, TRUE);
-
   return false;
 }
 
@@ -455,15 +451,17 @@ bool cmd_search(girara_session_t* session, const char* input, girara_argument_t*
     }
   }
 
-  girara_argument_t* arg = g_try_malloc0(sizeof(girara_argument_t));
+  g_autofree girara_argument_t* arg = g_try_malloc0(sizeof(girara_argument_t));
   if (arg == NULL) {
     return false;
   }
 
+  bool inc_search = false;
+  girara_setting_get(session, "incremental-search", &inc_search);
+
   arg->n    = FORWARD;
   arg->data = (void*)input;
-  sc_search(session, arg, NULL, 0);
-  g_free(arg);
+  search_document(zathura, arg, inc_search);
 
   return true;
 }
@@ -495,13 +493,13 @@ bool cmd_export(girara_session_t* session, girara_list_t* argument_list) {
     return false;
   }
 
-  char* export_path = girara_fix_path(file_name);
+  g_autofree char* export_path = girara_fix_path(file_name);
   if (export_path == NULL) {
     return false;
   }
 
   static const size_t attachment_len = 11; // length of attachment-
-  static const size_t image_len = 7; // length of image-p
+  static const size_t image_len      = 7;  // length of image-p
 
   /* attachment */
   if (strncmp(file_identifier, "attachment-", attachment_len) == 0) {
@@ -557,21 +555,16 @@ bool cmd_export(girara_session_t* session, girara_list_t* argument_list) {
       girara_notify(session, GIRARA_ERROR, _("Couldn't write image '%s' to '%s'."), file_identifier, file_name);
     }
 
-    goto error_ret;
+    return true;
 
   image_error:
-
     girara_notify(session, GIRARA_ERROR, _("Unknown image '%s'."), file_identifier);
-    goto error_ret;
+    return true;
     /* unknown */
   } else {
     girara_notify(session, GIRARA_ERROR, _("Unknown attachment or image '%s'."), file_identifier);
   }
 
-error_ret:
-
-  g_free(export_path);
-
   return true;
 #endif
 }
@@ -604,12 +597,10 @@ bool cmd_exec(girara_session_t* session, girara_list_t* argument_list) {
     g_ascii_dtostr(page_buf, G_ASCII_DTOSTR_BUF_SIZE, page + 1);
 
     for (size_t idx = 0; idx != girara_list_size(argument_list); ++idx) {
-      char* value = girara_list_nth(argument_list, idx);
-      char* r     = girara_replace_substring(value, "$PAGE", page_buf);
+      char* value        = girara_list_nth(argument_list, idx);
+      g_autofree char* r = girara_replace_substring(value, "$PAGE", page_buf);
       if (r != NULL) {
         char* s = girara_replace_substring(r, "$FILE", path);
-        g_free(r);
-
         if (s != NULL) {
           girara_list_set_nth(argument_list, idx, s);
         }
@@ -656,16 +647,13 @@ bool cmd_version(girara_session_t* session, girara_list_t* UNUSED(argument_list)
   g_return_val_if_fail(session->global.data != NULL, false);
   zathura_t* zathura = session->global.data;
 
-  char* string = zathura_get_version_string(zathura->plugins.manager, true);
+  g_autofree char* string = zathura_get_version_string(zathura->plugins.manager, true);
   if (string == NULL) {
     return false;
   }
 
   /* display information */
   girara_notify(session, GIRARA_INFO, "%s", string);
-
-  g_free(string);
-
   return true;
 }
 
