@@ -14,19 +14,20 @@
 #include "page.h"
 #include "database.h"
 
-#include <girara/session.h>
-#include <girara/settings.h>
-#include <girara/completion.h>
-#include <girara/utils.h>
+#include <girara-gtk/session.h>
+#include <girara-gtk/settings.h>
+#include <girara-gtk/completion.h>
 #include <girara/datastructures.h>
+#include <girara/log.h>
+#include <girara/utils.h>
 
-static int compare_case_insensitive(const char* str1, const char* str2) {
-  char* ustr1 = g_utf8_casefold(str1, -1);
-  char* ustr2 = g_utf8_casefold(str2, -1);
-  int res     = g_utf8_collate(ustr1, ustr2);
-  g_free(ustr1);
-  g_free(ustr2);
-  return res;
+static int compare_case_insensitive(const void* data1, const void* data2) {
+  const char* str1 = data1;
+  const char* str2 = data2;
+
+  g_autofree char* ustr1 = g_utf8_casefold(str1, -1);
+  g_autofree char* ustr2 = g_utf8_casefold(str2, -1);
+  return g_utf8_collate(ustr1, ustr2);
 }
 
 static girara_list_t* list_files(zathura_t* zathura, const char* current_path, const char* current_file,
@@ -38,13 +39,12 @@ static girara_list_t* list_files(zathura_t* zathura, const char* current_path, c
   girara_debug("checking files in %s", current_path);
 
   /* read directory */
-  GDir* dir = g_dir_open(current_path, 0, NULL);
+  g_autoptr(GDir) dir = g_dir_open(current_path, 0, NULL);
   if (dir == NULL) {
     return NULL;
   }
 
-  girara_list_t* res =
-      girara_sorted_list_new2((girara_compare_function_t)compare_case_insensitive, (girara_free_function_t)g_free);
+  girara_list_t* res = girara_sorted_list_new_with_free(compare_case_insensitive, g_free);
 
   bool show_hidden = false;
   girara_setting_get(zathura->ui.session, "show-hidden", &show_hidden);
@@ -54,7 +54,7 @@ static girara_list_t* list_files(zathura_t* zathura, const char* current_path, c
   /* read files */
   const char* name = NULL;
   while ((name = g_dir_read_name(dir)) != NULL) {
-    char* e_name = g_filename_display_name(name);
+    g_autofree char* e_name = g_filename_display_name(name);
     if (e_name == NULL) {
       goto error_free;
     }
@@ -62,42 +62,37 @@ static girara_list_t* list_files(zathura_t* zathura, const char* current_path, c
     size_t e_length = strlen(e_name);
 
     if (show_hidden == false && e_name[0] == '.') {
-      g_free(e_name);
       continue;
     }
 
     if ((current_file_length > e_length) || strncmp(current_file, e_name, current_file_length)) {
-      g_free(e_name);
       continue;
     }
 
-    char* tmp = "/";
+    const char* tmp = "/";
     if (is_dir == true || g_strcmp0(current_path, "/") == 0) {
       tmp = "";
     }
 
-    char* full_path = g_strdup_printf("%s%s%s", current_path, tmp, e_name);
-    g_free(e_name);
+    g_autofree char* full_path = g_strdup_printf("%s%s%s", current_path, tmp, e_name);
 
     if (g_file_test(full_path, G_FILE_TEST_IS_DIR) == true) {
       if (show_directories == false) {
         girara_debug("ignoring %s (directory)", full_path);
-        g_free(full_path);
         continue;
       }
       girara_debug("adding %s (directory)", full_path);
       girara_list_append(res, full_path);
+      full_path = NULL;
     } else if (check_file_ext == false || file_valid_extension(zathura, full_path) == true) {
       girara_debug("adding %s (file)", full_path);
       girara_list_append(res, full_path);
+      full_path = NULL;
     } else {
       girara_debug("ignoring %s (file)", full_path);
-      g_free(full_path);
     }
   }
 
-  g_dir_close(dir);
-
   if (girara_list_size(res) == 1) {
     char* path = girara_list_nth(res, 0);
     if (g_file_test(path, G_FILE_TEST_IS_DIR) == true) {
@@ -111,7 +106,6 @@ static girara_list_t* list_files(zathura_t* zathura, const char* current_path, c
   return res;
 
 error_free:
-  g_dir_close(dir);
   girara_list_free(res);
   return NULL;
 }
@@ -128,9 +122,8 @@ static girara_completion_t* list_files_for_cc(zathura_t* zathura, const char* in
   girara_completion_t* completion          = girara_completion_init();
   girara_completion_group_t* group         = girara_completion_group_create(zathura->ui.session, "files");
   girara_completion_group_t* history_group = NULL;
-
-  gchar* path         = NULL;
-  gchar* current_path = NULL;
+  g_autofree gchar* path                   = NULL;
+  g_autofree gchar* current_path           = NULL;
 
   if (show_recent > 0) {
     history_group = girara_completion_group_create(zathura->ui.session, "recent files");
@@ -148,13 +141,12 @@ static girara_completion_t* list_files_for_cc(zathura_t* zathura, const char* in
   /* If the path does not begin with a slash we update the path with the current
    * working directory */
   if (strlen(path) == 0 || path[0] != '/') {
-    char* cwd = g_get_current_dir();
+    g_autofree char* cwd = g_get_current_dir();
     if (cwd == NULL) {
       goto error_free;
     }
 
     char* tmp_path = g_strdup_printf("%s/%s", cwd, path);
-    g_free(cwd);
     if (tmp_path == NULL) {
       goto error_free;
     }
@@ -181,33 +173,29 @@ static girara_completion_t* list_files_for_cc(zathura_t* zathura, const char* in
 
   /* read directory */
   if (g_file_test(current_path, G_FILE_TEST_IS_DIR) == TRUE) {
-    girara_list_t* names = list_files(zathura, current_path, current_file, current_file_length, is_dir, check_file_ext);
+    g_autoptr(girara_list_t) names =
+        list_files(zathura, current_path, current_file, current_file_length, is_dir, check_file_ext);
     if (names == NULL) {
       goto error_free;
     }
 
     girara_list_foreach(names, group_add_element, group);
-    girara_list_free(names);
   }
 
-  if (show_recent > 0 && zathura->database != NULL) {
-    girara_list_t* recent_files = zathura_db_get_recent_files(zathura->database, show_recent, path);
+  if (show_recent > 0) {
+    g_autoptr(girara_list_t) recent_files = zathura_db_get_recent_files(zathura->database, show_recent, path);
     if (recent_files == NULL) {
       goto error_free;
     }
 
     if (girara_list_size(recent_files) != 0) {
       girara_list_foreach(recent_files, group_add_element, history_group);
-      girara_list_free(recent_files);
     } else {
       girara_completion_group_free(history_group);
       history_group = NULL;
     }
   }
 
-  g_free(path);
-  g_free(current_path);
-
   if (history_group != NULL) {
     girara_completion_add_group(completion, history_group);
   }
@@ -216,7 +204,6 @@ static girara_completion_t* list_files_for_cc(zathura_t* zathura, const char* in
   return completion;
 
 error_free:
-
   if (completion) {
     girara_completion_free(completion);
   }
@@ -227,9 +214,6 @@ error_free:
     girara_completion_group_free(group);
   }
 
-  g_free(current_path);
-  g_free(path);
-
   return NULL;
 }
 
@@ -272,9 +256,8 @@ girara_completion_t* cc_bookmarks(girara_session_t* session, const char* input)
   for (size_t idx = 0; idx != girara_list_size(zathura->bookmarks.bookmarks); ++idx) {
     zathura_bookmark_t* bookmark = girara_list_nth(zathura->bookmarks.bookmarks, idx);
     if (input_length <= strlen(bookmark->id) && !strncmp(input, bookmark->id, input_length)) {
-      gchar* paged = g_strdup_printf(_("Page %d"), bookmark->page);
+      g_autofree gchar* paged = g_strdup_printf(_("Page %d"), bookmark->page);
       girara_completion_group_add_element(group, bookmark->id, paged);
-      g_free(paged);
     }
   }
 
@@ -283,7 +266,6 @@ girara_completion_t* cc_bookmarks(girara_session_t* session, const char* input)
   return completion;
 
 error_free:
-
   if (completion != NULL) {
     girara_completion_free(completion);
   }
@@ -302,16 +284,17 @@ girara_completion_t* cc_export(girara_session_t* session, const char* input) {
   zathura_document_t* document = zathura_get_document(zathura);
 
   if (input == NULL || document == NULL) {
-    goto error_ret;
+    return NULL;
   }
 
   girara_completion_t* completion             = NULL;
   girara_completion_group_t* attachment_group = NULL;
   girara_completion_group_t* image_group      = NULL;
+  g_autoptr(girara_list_t) attachments        = NULL;
 
   completion = girara_completion_init();
   if (completion == NULL) {
-    goto error_free;
+    return NULL;
   }
 
   attachment_group = girara_completion_group_create(session, _("Attachments"));
@@ -320,17 +303,16 @@ girara_completion_t* cc_export(girara_session_t* session, const char* input) {
   }
 
   /* add attachments */
-  const size_t input_length = strlen(input);
-  zathura_error_t attachment_error;
-  girara_list_t* attachments = zathura_document_attachments_get(document, &attachment_error);
-  bool added_attachment = false;
+  const size_t input_length        = strlen(input);
+  zathura_error_t attachment_error = ZATHURA_ERROR_OK;
+  attachments                      = zathura_document_attachments_get(document, &attachment_error);
+  bool added_attachment            = false;
   if (attachments != NULL) {
     for (size_t idx = 0; idx != girara_list_size(attachments); ++idx) {
       const char* attachment = girara_list_nth(attachments, idx);
       if (input_length <= strlen(attachment) && !strncmp(input, attachment, input_length)) {
-        char* attachment_string = g_strdup_printf("attachment-%s", attachment);
+        g_autofree char* attachment_string = g_strdup_printf("attachment-%s", attachment);
         girara_completion_group_add_element(attachment_group, attachment_string, NULL);
-        g_free(attachment_string);
         added_attachment = true;
       }
     }
@@ -341,8 +323,6 @@ girara_completion_t* cc_export(girara_session_t* session, const char* input) {
       girara_completion_group_free(attachment_group);
       attachment_group = NULL;
     }
-
-    girara_list_free(attachments);
   }
 
   /* add images */
@@ -353,25 +333,23 @@ girara_completion_t* cc_export(girara_session_t* session, const char* input) {
 
   bool added_image                   = false;
   const unsigned int number_of_pages = zathura_document_get_number_of_pages(document);
-  zathura_error_t image_error;
+  zathura_error_t image_error        = ZATHURA_ERROR_OK;
   for (unsigned int page_id = 0; page_id < number_of_pages; page_id++) {
     zathura_page_t* page = zathura_document_get_page(document, page_id);
     if (page == NULL) {
       continue;
     }
 
-    girara_list_t* images = zathura_page_images_get(page, &image_error);
+    g_autoptr(girara_list_t) images = zathura_page_images_get(page, &image_error);
     if (images != NULL) {
       unsigned int image_number = 1;
       for (size_t idx = 0; idx != girara_list_size(images); ++idx) {
-        char* image_string = g_strdup_printf("image-p%d-%d", page_id + 1, image_number);
+        g_autofree char* image_string = g_strdup_printf("image-p%d-%d", page_id + 1, image_number);
         girara_completion_group_add_element(image_group, image_string, NULL);
-        g_free(image_string);
 
         added_image = true;
         image_number++;
       }
-      girara_list_free(images);
     } else if (image_error == ZATHURA_ERROR_NOT_IMPLEMENTED) {
       break;
     }
@@ -396,7 +374,6 @@ girara_completion_t* cc_export(girara_session_t* session, const char* input) {
   return completion;
 
 error_free:
-
   if (completion != NULL) {
     girara_completion_free(completion);
   }
@@ -409,7 +386,5 @@ error_free:
     girara_completion_group_free(image_group);
   }
 
-error_ret:
-
   return NULL;
 }
