@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <girara/log.h>
 #include <girara/utils.h>
 
 #include "landlock.h"
@@ -57,8 +58,7 @@ static void landlock_drop(__u64 fs_access) {
 #define _LANDLOCK_ACCESS_FS_READ (LANDLOCK_ACCESS_FS_READ_FILE | LANDLOCK_ACCESS_FS_READ_DIR)
 
 static void landlock_check_kernel(void) {
-  int abi = landlock_create_ruleset(NULL, 0,
-                                  LANDLOCK_CREATE_RULESET_VERSION);
+  int abi = landlock_create_ruleset(NULL, 0, LANDLOCK_CREATE_RULESET_VERSION);
   if (abi == -1) {
     /*
      * Kernel too old, not compiled with Landlock,
@@ -66,7 +66,7 @@ static void landlock_check_kernel(void) {
      */
     girara_warning("Unable to use Landlock: Kernel too old, not compiled with Landlock,\
             or Landlock was not enabled at boot time. Sandbox partly disabled.");
-    return;  /* Graceful fallback: Do nothing. */
+    return; /* Graceful fallback: Do nothing. */
   }
 }
 
@@ -75,11 +75,13 @@ void landlock_drop_write(void) {
   landlock_drop(_LANDLOCK_ACCESS_FS_WRITE | LANDLOCK_ACCESS_FS_EXECUTE);
 }
 
-void landlock_drop_all(void) {
+#if 0
+static void landlock_drop_all(void) {
   landlock_drop(_LANDLOCK_ACCESS_FS_READ | _LANDLOCK_ACCESS_FS_WRITE | LANDLOCK_ACCESS_FS_EXECUTE);
 }
+#endif
 
-void landlock_write_fd(const int dir_fd) {
+static void landlock_write_fd(const int dir_fd) {
   const struct landlock_ruleset_attr ruleset_attr = {
       .handled_access_fs = _LANDLOCK_ACCESS_FS_WRITE | LANDLOCK_ACCESS_FS_EXECUTE,
   };
@@ -114,8 +116,8 @@ void landlock_write_fd(const int dir_fd) {
 
 void landlock_restrict_write(void) {
   landlock_check_kernel();
-  char* data_path = girara_get_xdg_path(XDG_DATA);
-  int fd          = open(data_path, O_PATH | O_CLOEXEC | O_DIRECTORY);
-  g_free(data_path);
+  g_autofree char* data_path = girara_get_xdg_path(XDG_DATA);
+  int fd                     = open(data_path, O_PATH | O_CLOEXEC | O_DIRECTORY);
   landlock_write_fd(fd);
+  close(fd);
 }
