@@ -11,6 +11,7 @@
 #include "adjustment.h"
 #include "zathura.h"
 #include "document.h"
+#include "document-widget.h"
 #include "page.h"
 #include "page-widget.h"
 #include "utils.h"
@@ -39,6 +40,7 @@ typedef struct private_s {
     bool enabled;
     bool hue;
     bool reverse_video;
+    bool adjust_lightness;
   } recolor;
 
   atomic_bool about_to_close; /**< Render thread is to be freed */
@@ -93,9 +95,10 @@ static void zathura_renderer_init(ZathuraRenderer* renderer) {
   g_mutex_init(&priv->mutex);
 
   /* recolor */
-  priv->recolor.enabled       = false;
-  priv->recolor.hue           = true;
-  priv->recolor.reverse_video = false;
+  priv->recolor.enabled          = false;
+  priv->recolor.hue              = true;
+  priv->recolor.reverse_video    = false;
+  priv->recolor.adjust_lightness = false;
 
   /* page cache */
   priv->page_cache.size             = 0;
@@ -291,6 +294,21 @@ void zathura_renderer_enable_recolor_reverse_video(ZathuraRenderer* renderer, bo
   ZathuraRendererPrivate* priv = zathura_renderer_get_instance_private(renderer);
   priv->recolor.reverse_video  = enable;
 }
+
+bool zathura_renderer_recolor_adjust_lightness_enabled(ZathuraRenderer* renderer) {
+  g_return_val_if_fail(ZATHURA_IS_RENDERER(renderer), false);
+
+  ZathuraRendererPrivate* priv = zathura_renderer_get_instance_private(renderer);
+  return priv->recolor.adjust_lightness;
+}
+
+void zathura_renderer_enable_recolor_adjust_lightness(ZathuraRenderer* renderer, bool enable) {
+  g_return_if_fail(ZATHURA_IS_RENDERER(renderer));
+
+  ZathuraRendererPrivate* priv   = zathura_renderer_get_instance_private(renderer);
+  priv->recolor.adjust_lightness = enable;
+}
+
 void zathura_renderer_set_recolor_colors(ZathuraRenderer* renderer, const GdkRGBA* light, const GdkRGBA* dark) {
   g_return_if_fail(ZATHURA_IS_RENDERER(renderer));
 
@@ -518,6 +536,8 @@ static void recolor_slow(ZathuraRendererPrivate* priv, unsigned int page_width,
       rgb2.blue * rgb2.alpha - l2,
   };
 
+  bool adjust_lightness = priv->recolor.adjust_lightness;
+
   const int rowstride  = cairo_image_surface_get_stride(surface);
   unsigned char* image = cairo_image_surface_get_data(surface);
 
@@ -552,6 +572,14 @@ static void recolor_slow(ZathuraRendererPrivate* priv, unsigned int page_width,
         const double u = colorumax(h, l, 0, 1);
         const double s = fabs(u) > DBL_EPSILON ? 1.0 / u : 0.0;
 
+        /* adjust according to quartic curve, then average with original weighed
+         * by half saturation. */
+        if (adjust_lightness) {
+          /* l = l * s/2 + l^4 * (1 - s/2) */
+          double adj = l * l * l * l;
+          l          = (l - adj) * (s * 0.5) + adj;
+        }
+
         /* Interpolates lightness between light and dark colors. white goes to
          * light, and black goes to dark. */
         l = l * (l2 - l1) + l1;
@@ -567,9 +595,12 @@ static void recolor_slow(ZathuraRendererPrivate* priv, unsigned int page_width,
         data[1]          = (unsigned char)round(255. * fmin(1, fmax(0, tr1 * h1[1] + tr2 * h2[1] + (l + su * h[1]))));
         data[0]          = (unsigned char)round(255. * fmin(1, fmax(0, tr1 * h1[2] + tr2 * h2[2] + (l + su * h[2]))));
       } else {
+        if (adjust_lightness) {
+          l = l * l;
+        }
+
         /* linear interpolation between dark and light with color ligtness as
          * a parameter */
-
         const double f1 = 1. - (1. - fmax(fmax(rgb[0], rgb[1]), rgb[2])) * negalph1;
         const double f2 = fmin(fmin(rgb[0], rgb[1]), rgb[2]) * negalph2;
         data[3]         = (unsigned char)round(255. * (f1 - f2));
@@ -591,6 +622,8 @@ static void recolor_fast(ZathuraRendererPrivate* priv, unsigned int page_width,
 
   const double rgb_diff[] = {rgb2.red - rgb1.red, rgb2.green - rgb1.green, rgb2.blue - rgb1.blue};
 
+  bool adjust_lightness = priv->recolor.adjust_lightness;
+
   const int rowstride  = cairo_image_surface_get_stride(surface);
   unsigned char* image = cairo_image_surface_get_data(surface);
 
@@ -625,6 +658,14 @@ static void recolor_fast(ZathuraRendererPrivate* priv, unsigned int page_width,
         const double u = colorumax(h, l, 0, 1);
         const double s = fabs(u) > DBL_EPSILON ? 1.0 / u : 0.0;
 
+        /* adjust according to quartic curve, then average with original weighed
+         * by half saturation. */
+        if (adjust_lightness) {
+          /* l = l * s/2 + l^4 * (1 - s/2) */
+          double adj = l * l * l * l;
+          l          = (l - adj) * (s * 0.5) + adj;
+        }
+
         /* Interpolates lightness between light and dark colors. white goes to
          * light, and black goes to dark. */
         l = l * (l2 - l1) + l1;
@@ -638,6 +679,10 @@ static void recolor_fast(ZathuraRendererPrivate* priv, unsigned int page_width,
         data[1] = (unsigned char)round(255. * (l + su * h[1]));
         data[0] = (unsigned char)round(255. * (l + su * h[2]));
       } else {
+        if (adjust_lightness) {
+          l = l * l;
+        }
+
         /* linear interpolation between dark and light with color ligtness as
          * a parameter */
         data[3] = 255;
@@ -650,7 +695,7 @@ static void recolor_fast(ZathuraRendererPrivate* priv, unsigned int page_width,
 }
 
 static void recolor(ZathuraRendererPrivate* priv, zathura_page_t* page, unsigned int page_width,
-                    unsigned int page_height, cairo_surface_t* surface) {
+                    unsigned int page_height, cairo_surface_t* surface, zathura_device_factors_t device_factors) {
   /* uses a representation of a rgb color as follows:
      - a lightness scalar (between 0,1), which is a weighted average of r, g, b,
      - a hue vector, which indicates a radian direction from the grey axis,
@@ -676,13 +721,13 @@ static void recolor(ZathuraRendererPrivate* priv, zathura_page_t* page, unsigned
                               fabs(rgb2.red - rgb2.blue) < DBL_EPSILON && fabs(rgb2.red - rgb2.green) < DBL_EPSILON)) &&
       (rgb1.alpha >= 1. - DBL_EPSILON && rgb2.alpha >= 1. - DBL_EPSILON);
 
-  girara_list_t* rectangles = NULL;
-  bool found_images         = false;
+  g_autoptr(girara_list_t) rectangles = NULL;
+  bool found_images                   = false;
 
   /* If in reverse video mode retrieve images */
   if (priv->recolor.reverse_video == true) {
-    girara_list_t* images = zathura_page_images_get(page, NULL);
-    found_images          = (images != NULL);
+    g_autoptr(girara_list_t) images = zathura_page_images_get(page, NULL);
+    found_images                    = (images != NULL);
 
     rectangles = girara_list_new();
     if (rectangles == NULL) {
@@ -699,13 +744,14 @@ static void recolor(ZathuraRendererPrivate* priv, zathura_page_t* page, unsigned
           break;
         }
         *rect = recalc_rectangle(page, image_it->position);
+        /* Scale rectangle coordinates by device factors to match surface pixel coordinates */
+        rect->x1 *= device_factors.x;
+        rect->x2 *= device_factors.x;
+        rect->y1 *= device_factors.y;
+        rect->y2 *= device_factors.y;
         girara_list_append(rectangles, rect);
       }
     }
-
-    if (images != NULL) {
-      girara_list_free(images);
-    }
   }
 
   if (fast_formula == true) {
@@ -714,10 +760,6 @@ static void recolor(ZathuraRendererPrivate* priv, zathura_page_t* page, unsigned
     recolor_slow(priv, page_width, page_height, surface, rectangles, found_images);
   }
 
-  if (rectangles != NULL) {
-    girara_list_free(rectangles);
-  }
-
   cairo_surface_mark_dirty(surface);
 }
 
@@ -780,7 +822,7 @@ static bool render(render_job_t* job, ZathuraRenderRequest* request, ZathuraRend
     /* page size in user pixels based on document zoom: if PPI information is
      * correct, 100% zoom will result in 72 documents points per inch of screen
      * (i.e. document size on screen matching the physical paper size). */
-    real_scale = page_calc_height_width(document, height, width, &page_height, &page_width, false);
+    real_scale = page_calc_height_width(document, page, &page_height, &page_width, false);
 
     device_factors = zathura_document_get_device_factors(document);
     page_width *= device_factors.x;
@@ -822,7 +864,7 @@ static bool render(render_job_t* job, ZathuraRenderRequest* request, ZathuraRend
 
   /* recolor */
   if (request_priv->render_plain == false && priv->recolor.enabled == true) {
-    recolor(priv, page, page_width, page_height, surface);
+    recolor(priv, page, page_width, page_height, surface, device_factors);
   }
 
   if (!invoke_completed_signal(job, surface)) {
@@ -863,17 +905,17 @@ void render_all(zathura_t* zathura) {
     return;
   }
 
+  zathura_document_widget_compute_layout(ZATHURA_DOCUMENT_WIDGET(zathura->ui.document_widget));
+
   /* unmark all pages */
   const unsigned int number_of_pages = zathura_document_get_number_of_pages(document);
   for (unsigned int page_id = 0; page_id < number_of_pages; ++page_id) {
     zathura_page_t* page     = zathura_document_get_page(document, page_id);
     unsigned int page_height = 0, page_width = 0;
-    const double height = zathura_page_get_height(page);
-    const double width  = zathura_page_get_width(page);
-    page_calc_height_width(document, height, width, &page_height, &page_width, true);
 
-    girara_debug("Queuing resize for page %u to %u x %u (%0.2f x %0.2f).", page_id, page_width, page_height, width,
-                 height);
+    page_calc_height_width(document, page, &page_height, &page_width, true);
+
+    girara_debug("Queuing resize for page %u to %u x %u.", page_id, page_width, page_height);
     GtkWidget* widget = zathura_page_get_widget(zathura, page);
     if (widget != NULL) {
       gtk_widget_set_size_request(widget, page_width, page_height);
