@@ -1,27 +1,31 @@
 /* SPDX-License-Identifier: Zlib */
 
-#include <girara/log.h>
-#include <girara/session.h>
-#include <girara/settings.h>
-#include <girara/datastructures.h>
-#include <girara/shortcuts.h>
-#include <girara/utils.h>
-#include <gtk/gtk.h>
-#include <glib/gi18n.h>
-
-#include "callbacks.h"
 #include "shortcuts.h"
+
+#include <girara-gtk/internal.h>
+#include <girara-gtk/session.h>
+#include <girara-gtk/settings.h>
+#include <girara-gtk/shortcuts.h>
+#include <girara/datastructures.h>
+#include <girara/log.h>
+#include <girara/utils.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <math.h>
+
+#include "adjustment.h"
+#include "callbacks.h"
+#include "commands.h"
+#include "database.h"
 #include "dbus-interface.h"
+#include "document-widget.h"
 #include "document.h"
-#include "zathura.h"
-#include "render.h"
-#include "utils.h"
+#include "page-widget.h"
 #include "page.h"
 #include "print.h"
-#include "page-widget.h"
-#include "adjustment.h"
-#include "database.h"
-#include <math.h>
+#include "render.h"
+#include "utils.h"
+#include "zathura.h"
 
 /* Helper function for highlighting the links */
 static bool draw_links(zathura_t* zathura) {
@@ -196,7 +200,7 @@ bool sc_copy_filepath(girara_session_t* session, girara_argument_t* UNUSED(argum
   g_return_val_if_fail(session->global.data != NULL, false);
   zathura_t* zathura = session->global.data;
 
-  GdkAtom* selection = get_selection(zathura);
+  g_autofree GdkAtom* selection = get_selection(zathura);
   if (selection == NULL) {
     return false;
   }
@@ -210,7 +214,60 @@ bool sc_copy_filepath(girara_session_t* session, girara_argument_t* UNUSED(argum
   girara_debug("Copying file path to clipboard");
   gtk_clipboard_set_text(gtk_clipboard_get(*selection), file_path, -1);
 
-  g_free(selection);
+  return true;
+}
+
+bool sc_equal_page_mode(girara_session_t* session, girara_argument_t* argument, girara_event_t* UNUSED(event),
+                        unsigned int UNUSED(t)) {
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t* zathura = session->global.data;
+  g_return_val_if_fail(argument != NULL, false);
+  g_return_val_if_fail(zathura->document != NULL, false);
+
+  if (argument->n >= ZATHURA_EQUAL_MODE_NUMBER) {
+    girara_error("equal mode: unknown mode %d", argument->n);
+    return false;
+  }
+
+  const unsigned int npag         = zathura_document_get_number_of_pages(zathura->document);
+  const unsigned int current_page = zathura_document_get_current_page_number(zathura->document);
+
+  zathura_page_t* c_page = zathura_document_get_page(zathura->document, current_page);
+  zathura_page_set_zoom(c_page, 1.0);
+
+  girara_debug("Setting page equal mode to: %d", argument->n);
+
+  if (argument->n == ZATHURA_EQUAL_NONE) {
+    /* reset page zooms */
+    for (unsigned int i = 0; i < npag; i++) {
+      zathura_page_t* p = zathura_document_get_page(zathura->document, i);
+      zathura_page_set_zoom(p, 1.0);
+    }
+
+    render_all(zathura);
+    refresh_view(zathura);
+    return true;
+  }
+
+  for (unsigned int i = 0; i != npag; i++) {
+    zathura_page_t* page_i = zathura_document_get_page(zathura->document, i);
+
+    zathura_page_set_zoom(page_i, 1.0);
+
+    switch (argument->n) {
+    case ZATHURA_EQUAL_WIDTH:
+      zathura_page_set_zoom(page_i, zathura_page_get_width(c_page) / zathura_page_get_width(page_i));
+      break;
+    case ZATHURA_EQUAL_HEIGHT:
+      zathura_page_set_zoom(page_i, zathura_page_get_height(c_page) / zathura_page_get_height(page_i));
+      break;
+    }
+  }
+
+  render_all(zathura);
+  refresh_view(zathura);
+
   return true;
 }
 
@@ -254,10 +311,10 @@ bool sc_focus_inputbar(girara_session_t* session, girara_argument_t* argument, g
       g_free(tmp);
     }
 
-    GdkAtom* selection = get_selection(zathura);
+    g_autofree GdkAtom* selection = get_selection(zathura);
 
     /* we save the X clipboard that will be clear by "grab_focus" */
-    gchar* x_clipboard_text = NULL;
+    g_autofree gchar* x_clipboard_text = NULL;
 
     if (selection != NULL) {
       x_clipboard_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(*selection));
@@ -268,10 +325,7 @@ bool sc_focus_inputbar(girara_session_t* session, girara_argument_t* argument, g
     if (x_clipboard_text != NULL && selection != NULL) {
       /* we reset the X clipboard with saved text */
       gtk_clipboard_set_text(gtk_clipboard_get(*selection), x_clipboard_text, -1);
-      g_free(x_clipboard_text);
     }
-
-    g_free(selection);
   }
 
   return true;
@@ -343,8 +397,8 @@ bool sc_mouse_scroll(girara_session_t* session, girara_argument_t* argument, gir
     zathura->shortcut.mouse.y = 0;
     break;
   case GIRARA_EVENT_MOTION_NOTIFY:
-    x_adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(session->gtk.view));
-    y_adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(session->gtk.view));
+    x_adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(zathura->ui.view));
+    y_adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(zathura->ui.view));
 
     if (x_adj == NULL || y_adj == NULL) {
       return false;
@@ -558,16 +612,29 @@ bool sc_scroll(girara_session_t* session, girara_argument_t* argument, girara_ev
   /* If PAGE_TOP or PAGE_BOTTOM, go there and we are done */
   if (argument->n == PAGE_TOP) {
     double dontcare = 0.5;
-    page_number_to_position(zathura->document, page_id, dontcare, 0.0, &dontcare, &pos_y);
+    page_number_to_position(zathura, page_id, dontcare, 0.0, &dontcare, &pos_y);
     position_set(zathura, pos_x, pos_y);
     return false;
   } else if (argument->n == PAGE_BOTTOM) {
     double dontcare = 0.5;
-    page_number_to_position(zathura->document, page_id, dontcare, 1.0, &dontcare, &pos_y);
+    page_number_to_position(zathura, page_id, dontcare, 1.0, &dontcare, &pos_y);
     position_set(zathura, pos_x, pos_y);
     return false;
   }
 
+  /* If SMOOTH_(UP|DOWN) , use GtkScrolledWindow signal */
+  if (argument->n == SMOOTH_UP) {
+    gboolean handled = FALSE;
+    g_signal_emit_by_name(G_OBJECT(zathura->ui.view), "scroll-child", GTK_SCROLL_STEP_BACKWARD, FALSE, &handled);
+
+    return false;
+  } else if (argument->n == SMOOTH_DOWN) {
+    gboolean handled = FALSE;
+    g_signal_emit_by_name(G_OBJECT(zathura->ui.view), "scroll-child", GTK_SCROLL_STEP_FORWARD, FALSE, &handled);
+
+    return false;
+  }
+
   if (t == 0) {
     t = 1;
   }
@@ -578,7 +645,8 @@ bool sc_scroll(girara_session_t* session, girara_argument_t* argument, girara_ev
 
   unsigned int doc_width  = 0;
   unsigned int doc_height = 0;
-  zathura_document_get_document_size(zathura->document, &doc_height, &doc_width);
+  zathura_document_widget_get_document_size(ZATHURA_DOCUMENT_WIDGET(zathura->ui.document_widget), &doc_height,
+                                            &doc_width);
 
   float scroll_step = 40;
   girara_setting_get(session, "scroll-step", &scroll_step);
@@ -602,9 +670,11 @@ bool sc_scroll(girara_session_t* session, girara_argument_t* argument, girara_ev
     direction = -1.0;
   }
 
-  unsigned int pad   = zathura_document_get_page_padding(zathura->document);
-  const double vstep = (double)(view_height + pad) / (double)doc_height;
-  const double hstep = (double)(view_width + pad) / (double)doc_width;
+  const unsigned int v_padding = zathura_document_get_page_v_padding(zathura->document);
+  const unsigned int h_padding = zathura_document_get_page_h_padding(zathura->document);
+
+  const double vstep = (double)(view_height + v_padding) / (double)doc_height;
+  const double hstep = (double)(view_width + h_padding) / (double)doc_width;
 
   /* compute new position */
   switch (argument->n) {
@@ -672,30 +742,30 @@ bool sc_scroll(girara_session_t* session, girara_argument_t* argument, girara_ev
   }
 
   /* snap to the border if we change page */
-  const unsigned int new_page_id = position_to_page_number(zathura->document, pos_x, pos_y);
+  const unsigned int new_page_id = position_to_page_number(zathura, pos_x, pos_y);
   if (scroll_page_aware == true && page_id != new_page_id) {
     double dummy = 0.0;
     switch (argument->n) {
     case FULL_LEFT:
     case HALF_LEFT:
-      page_number_to_position(zathura->document, new_page_id, 1.0, 0.0, &pos_x, &dummy);
+      page_number_to_position(zathura, new_page_id, 1.0, 0.0, &pos_x, &dummy);
       break;
 
     case FULL_RIGHT:
     case HALF_RIGHT:
-      page_number_to_position(zathura->document, new_page_id, 0.0, 0.0, &pos_x, &dummy);
+      page_number_to_position(zathura, new_page_id, 0.0, 0.0, &pos_x, &dummy);
       break;
 
     case FULL_UP:
     case HALF_UP:
     case PARTIAL_UP:
-      page_number_to_position(zathura->document, new_page_id, 0.0, 1.0, &dummy, &pos_y);
+      page_number_to_position(zathura, new_page_id, 0.0, 1.0, &dummy, &pos_y);
       break;
 
     case FULL_DOWN:
     case HALF_DOWN:
     case PARTIAL_DOWN:
-      page_number_to_position(zathura->document, new_page_id, 0.0, 0.0, &dummy, &pos_y);
+      page_number_to_position(zathura, new_page_id, 0.0, 0.0, &dummy, &pos_y);
       break;
     }
   }
@@ -863,135 +933,8 @@ bool sc_search(girara_session_t* session, girara_argument_t* argument, girara_ev
   g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(session->global.data != NULL, false);
   zathura_t* zathura = session->global.data;
-  g_return_val_if_fail(argument != NULL, false);
-  g_return_val_if_fail(zathura->document != NULL, false);
 
-  const unsigned int num_pages = zathura_document_get_number_of_pages(zathura->document);
-  const unsigned int cur_page  = zathura_document_get_current_page_number(zathura->document);
-  GtkWidget* cur_page_widget = zathura_page_get_widget(zathura, zathura_document_get_page(zathura->document, cur_page));
-  bool new_search            = argument->data != NULL;
-  bool nohlsearch            = false;
-  bool first_time_after_abort = false;
-
-  girara_setting_get(session, "nohlsearch", &nohlsearch);
-  if (nohlsearch == false) {
-    gboolean draw = FALSE;
-    g_object_get(G_OBJECT(cur_page_widget), "draw-search-results", &draw, NULL);
-
-    if (draw == false) {
-      first_time_after_abort = true;
-    }
-
-    document_draw_search_results(zathura, true);
-  }
-
-  int diff = argument->n == FORWARD ? 1 : -1;
-  if (zathura->global.search_direction == BACKWARD) {
-    diff = -diff;
-  }
-
-  zathura_page_t* target_page = NULL;
-  int target_idx              = 0;
-
-  for (unsigned int page_id = 0; page_id < num_pages; ++page_id) {
-    int tmp              = cur_page + diff * page_id;
-    zathura_page_t* page = zathura_document_get_page(zathura->document, (tmp + num_pages) % num_pages);
-    if (page == NULL) {
-      continue;
-    }
-
-    GtkWidget* page_widget = zathura_page_get_widget(zathura, page);
-
-    int num_search_results = 0, current = -1;
-    g_object_get(G_OBJECT(page_widget), "search-current", &current, "search-length", &num_search_results, NULL);
-    if (num_search_results == 0 || current == -1) {
-      continue;
-    }
-
-    if (new_search == true || first_time_after_abort == true || (tmp + num_pages) % num_pages != cur_page) {
-      target_page = page;
-      target_idx  = diff == 1 ? 0 : num_search_results - 1;
-      break;
-    }
-
-    if (diff == 1 && current < num_search_results - 1) {
-      /* the next result is on the same page */
-      target_page = page;
-      target_idx  = current + 1;
-    } else if (diff == -1 && current > 0) {
-      target_page = page;
-      target_idx  = current - 1;
-    } else {
-      /* the next result is on a different page */
-      g_object_set(G_OBJECT(page_widget), "search-current", -1, NULL);
-
-      for (unsigned int npage_id = 1; npage_id < num_pages; ++npage_id) {
-        int ntmp                     = cur_page + diff * (page_id + npage_id);
-        zathura_page_t* npage        = zathura_document_get_page(zathura->document, (ntmp + 2 * num_pages) % num_pages);
-        GtkWidget* npage_page_widget = zathura_page_get_widget(zathura, npage);
-        g_object_get(G_OBJECT(npage_page_widget), "search-length", &num_search_results, NULL);
-        if (num_search_results != 0) {
-          target_page = npage;
-          target_idx  = diff == 1 ? 0 : num_search_results - 1;
-          break;
-        }
-      }
-    }
-
-    break;
-  }
-
-  if (target_page != NULL) {
-    girara_list_t* results   = NULL;
-    GtkWidget* page_widget   = zathura_page_get_widget(zathura, target_page);
-    GObject* obj_page_widget = G_OBJECT(page_widget);
-    g_object_set(obj_page_widget, "search-current", target_idx, NULL);
-    g_object_get(obj_page_widget, "search-results", &results, NULL);
-
-    /* Need to adjust rectangle to page scale and orientation */
-    zathura_rectangle_t* rect     = girara_list_nth(results, target_idx);
-    zathura_rectangle_t rectangle = recalc_rectangle(target_page, *rect);
-
-    bool search_hadjust = true;
-    girara_setting_get(session, "search-hadjust", &search_hadjust);
-
-    /* compute the position of the center of the page */
-    double pos_x = 0;
-    double pos_y = 0;
-    page_number_to_position(zathura->document, zathura_page_get_index(target_page), 0.5, 0.5, &pos_x, &pos_y);
-
-    /* correction to center the current result                          */
-    /* NOTE: rectangle is in viewport units, already scaled and rotated */
-    unsigned int cell_height = 0;
-    unsigned int cell_width  = 0;
-    zathura_document_get_cell_size(zathura->document, &cell_height, &cell_width);
-
-    unsigned int doc_height = 0;
-    unsigned int doc_width  = 0;
-    zathura_document_get_document_size(zathura->document, &doc_height, &doc_width);
-
-    /* compute the center of the rectangle, which will be aligned to the center
-       of the viewport */
-    const double center_y = (rectangle.y1 + rectangle.y2) / 2;
-    pos_y += (center_y - (double)cell_height / 2) / (double)doc_height;
-
-    if (search_hadjust == true) {
-      const double center_x = (rectangle.x1 + rectangle.x2) / 2;
-      pos_x += (center_x - (double)cell_width / 2) / (double)doc_width;
-    }
-
-    /* move to position */
-    zathura_jumplist_add(zathura);
-    position_set(zathura, pos_x, pos_y);
-    zathura_jumplist_add(zathura);
-  } else if (argument->data != NULL) {
-    const char* input  = argument->data;
-    char* escaped_text = g_markup_printf_escaped(_("Pattern not found: %s"), input);
-    girara_notify(session, GIRARA_ERROR, "%s", escaped_text);
-    g_free(escaped_text);
-  }
-
-  return false;
+  return search_document(zathura, argument, false);
 }
 
 bool sc_navigate_index(girara_session_t* session, girara_argument_t* argument, girara_event_t* UNUSED(event),
@@ -1062,12 +1005,6 @@ bool sc_navigate_index(girara_session_t* session, girara_argument_t* argument, g
       }
     }
     break;
-  case COLLAPSE:
-    if (gtk_tree_view_collapse_row(tree_view, path) == FALSE && gtk_tree_path_get_depth(path) > 1) {
-      gtk_tree_path_up(path);
-      gtk_tree_view_collapse_row(tree_view, path);
-    }
-    break;
   case DOWN:
     for (int n = (t == 0 ? 1 : t); n > 0; n--) {
       if (gtk_tree_view_row_expanded(tree_view, path)) {
@@ -1110,10 +1047,27 @@ bool sc_navigate_index(girara_session_t* session, girara_argument_t* argument, g
       gtk_tree_path_down(path);
     }
     break;
+  case EXPAND_RECURSIVE:
+    if (gtk_tree_view_expand_row(tree_view, path, TRUE)) {
+      gtk_tree_path_down(path);
+    }
+    break;
   case EXPAND_ALL:
     gtk_tree_view_expand_all(tree_view);
     need_to_scroll = TRUE;
     break;
+  case COLLAPSE:
+    if (gtk_tree_view_collapse_row(tree_view, path) == FALSE && gtk_tree_path_get_depth(path) > 1) {
+      gtk_tree_path_up(path);
+      gtk_tree_view_collapse_row(tree_view, path);
+    }
+    break;
+  case COLLAPSE_RECURSIVE:
+    while (gtk_tree_path_get_depth(path) > 1) {
+      gtk_tree_path_up(path);
+    }
+    gtk_tree_view_collapse_row(tree_view, path);
+    break;
   case COLLAPSE_ALL:
     gtk_tree_view_collapse_all(tree_view);
     while (gtk_tree_path_get_depth(path) > 1) {
@@ -1157,7 +1111,6 @@ bool sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argume
     return false;
   }
 
-  girara_tree_node_t* document_index = NULL;
   GtkWidget* treeview                = NULL;
   GtkTreeModel* model                = NULL;
   GtkCellRenderer* renderer          = NULL;
@@ -1174,7 +1127,7 @@ bool sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argume
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(zathura->ui.index), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
     /* create index */
-    document_index = zathura_document_index_generate(zathura->document, NULL);
+    g_autoptr(girara_tree_node_t) document_index = zathura_document_index_generate(zathura->document, NULL);
     if (document_index == NULL) {
       girara_notify(session, GIRARA_WARNING, _("This document does not contain any index"));
       goto error_free;
@@ -1205,7 +1158,6 @@ bool sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argume
     }
 
     document_index_build(session, model, NULL, document_index);
-    girara_node_free(document_index);
 
     /* setup widget */
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeview), 0, "Title", renderer, "markup", 0, NULL);
@@ -1220,12 +1172,12 @@ bool sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argume
     gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeview), FALSE);
     g_signal_connect(G_OBJECT(treeview), "row-activated", G_CALLBACK(cb_index_row_activated), zathura);
 
+    gtk_widget_set_visible(treeview, true);
     gtk_container_add(GTK_CONTAINER(zathura->ui.index), treeview);
   }
 
-  if (gtk_widget_get_visible(GTK_WIDGET(zathura->ui.index))) {
-    girara_set_view(session, zathura->ui.document_widget);
-    gtk_widget_hide(GTK_WIDGET(zathura->ui.index));
+  if (girara_mode_get(session) == zathura->modes.index) {
+    girara_set_view(zathura->ui.session, zathura->ui.view);
     girara_mode_set(zathura->ui.session, zathura->modes.normal);
 
     /* refresh view */
@@ -1242,8 +1194,8 @@ bool sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argume
     }
 
     girara_set_view(session, zathura->ui.index);
-    GtkTreeView* tree_view = gtk_container_get_children(GTK_CONTAINER(zathura->ui.index))->data;
-    gtk_widget_grab_focus(GTK_WIDGET(tree_view));
+    // GtkTreeView* tree_view = gtk_container_get_children(GTK_CONTAINER(zathura->ui.index))->data;
+    // gtk_widget_grab_focus(GTK_WIDGET(tree_view));
     index_scroll_to_current_page(zathura);
     girara_mode_set(zathura->ui.session, zathura->modes.index);
   }
@@ -1257,10 +1209,6 @@ error_free:
     zathura->ui.index = NULL;
   }
 
-  if (document_index != NULL) {
-    girara_node_free(document_index);
-  }
-
 error_ret:
 
   return false;
@@ -1340,6 +1288,12 @@ bool sc_toggle_presentation(girara_session_t* session, girara_argument_t* UNUSED
     /* reset pages per row */
     girara_setting_set(session, "pages-per-row", &zathura->shortcut.toggle_presentation_mode.pages);
 
+    // reset layout mode
+    if (zathura->shortcut.toggle_presentation_mode.layout_mode != DOCUMENT_WIDGET_SINGLE) {
+      g_object_set(zathura->ui.document_widget, "layout-mode", zathura->shortcut.toggle_presentation_mode.layout_mode,
+                   NULL);
+    }
+
     /* reset first page column */
     if (zathura->shortcut.toggle_presentation_mode.first_page_column_list != NULL) {
       girara_setting_set(session, "first-page-column",
@@ -1376,12 +1330,19 @@ bool sc_toggle_presentation(girara_session_t* session, girara_argument_t* UNUSED
     girara_setting_get(session, "first-page-column",
                        &zathura->shortcut.toggle_presentation_mode.first_page_column_list);
 
+    /* back up zoom */
+    zathura->shortcut.toggle_presentation_mode.zoom = zathura_document_get_zoom(zathura->document);
+
+    // backup layout mode
+    g_object_get(zathura->ui.document_widget, "layout-mode", &zathura->shortcut.toggle_presentation_mode.layout_mode,
+                 NULL);
+
     /* set single view */
     int int_value = 1;
     girara_setting_set(session, "pages-per-row", &int_value);
-
-    /* back up zoom */
-    zathura->shortcut.toggle_presentation_mode.zoom = zathura_document_get_zoom(zathura->document);
+    if (zathura->shortcut.toggle_presentation_mode.layout_mode != DOCUMENT_WIDGET_SINGLE) {
+      g_object_set(zathura->ui.document_widget, "layout-mode", DOCUMENT_WIDGET_SINGLE, NULL);
+    }
 
     /* adjust window */
     girara_argument_t argument = {.n = ZATHURA_ADJUST_BESTFIT, .data = NULL};
@@ -1407,6 +1368,30 @@ bool sc_toggle_presentation(girara_session_t* session, girara_argument_t* UNUSED
   return false;
 }
 
+bool sc_toggle_single_page_mode(girara_session_t* session, girara_argument_t* UNUSED(argument),
+                                girara_event_t* UNUSED(event), unsigned int UNUSED(t)) {
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t* zathura = session->global.data;
+
+  if (zathura->document == NULL) {
+    girara_notify(session, GIRARA_WARNING, _("No document opened."));
+    return false;
+  }
+
+  document_widget_mode_t old_mode;
+  g_object_get(zathura->ui.document_widget, "layout-mode", &old_mode, NULL);
+  if (old_mode == DOCUMENT_WIDGET_SINGLE) {
+    g_object_set(zathura->ui.document_widget, "layout-mode", DOCUMENT_WIDGET_GRID, NULL);
+  } else {
+    const int pages_per_row = 1;
+    girara_setting_set(zathura->ui.session, "pages-per-row", &pages_per_row);
+    g_object_set(zathura->ui.document_widget, "layout-mode", DOCUMENT_WIDGET_SINGLE, NULL);
+  }
+
+  return true;
+}
+
 bool sc_quit(girara_session_t* session, girara_argument_t* UNUSED(argument), girara_event_t* UNUSED(event),
              unsigned int UNUSED(t)) {
   g_return_val_if_fail(session != NULL, false);
@@ -1478,7 +1463,25 @@ bool sc_zoom(girara_session_t* session, girara_argument_t* argument, girara_even
   return false;
 }
 
-bool sc_exec(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t) {
+static bool sc_exec_internal(girara_session_t* session, girara_argument_t* argument) {
+  if (argument == NULL || argument->data == NULL) {
+    return false;
+  }
+
+  /* create argument list */
+  g_autoptr(girara_list_t) argument_list = argument_to_argument_list(argument);
+  if (argument_list == NULL) {
+    return false;
+  }
+
+  /* call exec */
+  cmd_exec(session, argument_list);
+
+  return false;
+}
+
+bool sc_exec(girara_session_t* session, girara_argument_t* argument, girara_event_t* UNUSED(event),
+             unsigned int UNUSED(t)) {
   g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(session->global.data != NULL, false);
   zathura_t* zathura = session->global.data;
@@ -1518,11 +1521,73 @@ bool sc_exec(girara_session_t* session, girara_argument_t* argument, girara_even
     new_argument.data = s;
   }
 
-  const bool ret = girara_sc_exec(session, &new_argument, event, t);
+  const bool ret = sc_exec_internal(session, &new_argument);
   g_free(new_argument.data);
   return ret;
 }
 
+bool sc_zoom_page(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t) {
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t* zathura = session->global.data;
+  g_return_val_if_fail(argument != NULL, false);
+  g_return_val_if_fail(zathura->document != NULL, false);
+
+  zathura_document_set_adjust_mode(zathura->document, ZATHURA_ADJUST_NONE);
+
+  /* retrieve zoom step value */
+  int value = 1;
+  girara_setting_get(zathura->ui.session, "zoom-step", &value);
+
+  unsigned int current_page = zathura_document_get_current_page_number(zathura->document);
+  zathura_page_t* page      = zathura_document_get_page(zathura->document, current_page);
+
+  const int nt           = (t == 0) ? 1 : t;
+  const double zoom_step = 1.0 + value / 100.0 * nt;
+  const double old_zoom  = zathura_page_get_zoom(page);
+
+  /* specify new zoom value */
+  if (argument->n == ZOOM_IN) {
+    girara_debug("Increasing page %d zoom by %0.2f.", current_page, zoom_step - 1.0);
+    zathura_page_set_zoom(page, old_zoom * zoom_step);
+  } else if (argument->n == ZOOM_OUT) {
+    girara_debug("Decreasing page %d zoom by %0.2f.", current_page, zoom_step - 1.0);
+    zathura_page_set_zoom(page, old_zoom / zoom_step);
+  } else if (argument->n == ZOOM_SPECIFIC) {
+    if (t == 0) {
+      girara_debug("Setting page %d zoom to 1.", current_page);
+      zathura_page_set_zoom(page, 1.0);
+    } else {
+      girara_debug("Setting page %d zoom to %0.2f.", current_page, t / 100.0);
+      zathura_page_set_zoom(page, t / 100.0);
+    }
+  } else if (argument->n == ZOOM_SMOOTH) {
+    const double dy = (event != NULL) ? event->y : 1.0;
+    const double z  = pow(zoom_step, -dy);
+    girara_debug("Increasing page %d zoom by %0.2f.", current_page, z - 1.0);
+    zathura_page_set_zoom(page, old_zoom * z);
+  } else {
+    girara_debug("Setting page %d zoom to 1.", current_page);
+    zathura_page_set_zoom(page, 1.0);
+  }
+
+  /* zoom limitations */
+  const double zoom = zathura_page_get_zoom(page);
+  zathura_page_set_zoom(page, zathura_correct_zoom_value(session, zoom));
+
+  const double new_zoom = zathura_page_get_zoom(page);
+  if (fabs(new_zoom - old_zoom) <= DBL_EPSILON) {
+    girara_debug("New and old page %d zoom level are too close: %0.2f vs. %0.2f", current_page, new_zoom, old_zoom);
+    return false;
+  }
+
+  girara_debug("Re-rendering with page %d new zoom level %0.2f.", current_page, new_zoom);
+  render_all(zathura);
+  refresh_view(zathura);
+
+  return false;
+}
+
 bool sc_nohlsearch(girara_session_t* session, girara_argument_t* UNUSED(argument), girara_event_t* UNUSED(event),
                    unsigned int UNUSED(t)) {
   g_return_val_if_fail(session != NULL, false);
@@ -1573,7 +1638,7 @@ bool sc_file_chooser(girara_session_t* session, girara_argument_t* UNUSED(argume
   const gint res = gtk_native_dialog_run(GTK_NATIVE_DIALOG(native));
   if (res == GTK_RESPONSE_ACCEPT) {
     char* filename = gtk_file_chooser_get_filename(chooser);
-    if (!document_close(zathura, false)) {
+    if (zathura_has_document(zathura) && !document_close(zathura, false)) {
       g_free(filename);
       goto error;
     }
