@@ -3,7 +3,7 @@
 #ifndef SHORTCUTS_H
 #define SHORTCUTS_H
 
-#include <girara/types.h>
+#include <girara-gtk/types.h>
 
 /**
  * Abort the current action and return to normal mode
@@ -83,6 +83,17 @@ bool sc_copy_link(girara_session_t* session, girara_argument_t* argument, girara
  */
 bool sc_copy_filepath(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t);
 
+/**
+ * Set page equal mode
+ *
+ * @param session The used girara session
+ * @param argument The used argument
+ * @param event Girara event
+ * @param t Number of executions
+ * @return true if no error occurred otherwise false
+ */
+bool sc_equal_page_mode(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t);
+
 /**
  * Shortcut function to focus the inputbar
  *
@@ -295,6 +306,18 @@ bool sc_toggle_fullscreen(girara_session_t* session, girara_argument_t* argument
 bool sc_toggle_presentation(girara_session_t* session, girara_argument_t* argument, girara_event_t* event,
                             unsigned int t);
 
+/**
+ * Toggle single page mode
+ *
+ * @param session The used girara session
+ * @param argument The used argument
+ * @param event Girara event
+ * @param t Number of executions
+ * @return true if no error occurred otherwise false
+ */
+bool sc_toggle_single_page_mode(girara_session_t* session, girara_argument_t* argument, girara_event_t* event,
+                                unsigned int t);
+
 /**
  * Quit zathura
  *
@@ -317,6 +340,17 @@ bool sc_quit(girara_session_t* session, girara_argument_t* argument, girara_even
  */
 bool sc_zoom(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t);
 
+/**
+ * Change the zoom level for the current page
+ *
+ * @param session The used girara session
+ * @param argument The used argument
+ * @param event Girara event
+ * @param t Number of executions
+ * @return true if no error occurred otherwise false
+ */
+bool sc_zoom_page(girara_session_t* session, girara_argument_t* argument, girara_event_t* event, unsigned int t);
+
 /**
  * Run external command.
  *
